/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

/**
 *
 Notes of the author : 
 
 1) before dereferencing, the ds:signature MUST be marshaled (see sign() function 
 in DOMXMLSignauture. This marshalling is necessary for the dereferencing. For
 example, a reference may be an object in the same document => dereferencer use
 the getHere() function of DOMURIReference.
 
 */


package be.cardon.xades;

import be.cardon.dsig.DigesterOutputStream;
import be.cardon.dsig.dom.ApacheData;
import be.cardon.dsig.dom.DOMReference;
import be.cardon.dsig.dom.DOMTransform;
import be.cardon.dsig.dom.DOMXMLSignature;
import java.io.OutputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.Provider;
import java.util.Iterator;
import java.util.List;
import javax.xml.crypto.Data;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.NodeSetData;
import javax.xml.crypto.OctetStreamData;
import javax.xml.crypto.XMLCryptoContext;
import javax.xml.crypto.dom.DOMCryptoContext;
import javax.xml.crypto.dsig.CanonicalizationMethod;
import javax.xml.crypto.dsig.TransformException;
import javax.xml.crypto.dsig.TransformService;
import javax.xml.crypto.dsig.XMLSignContext;
import javax.xml.crypto.dsig.XMLSignature;
import javax.xml.crypto.dsig.XMLSignatureException;
import javax.xml.crypto.dsig.dom.DOMSignContext;
import org.apache.xml.security.signature.XMLSignatureInput;
import org.apache.xml.security.utils.UnsyncBufferedOutputStream;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMAllDataObjectsTimeStamp extends DOMXadESTimeStamp{
    
    public final static String TAG_ALLDATAOBJECTSTIMESTAMP = "AllDataObjectsTimeStamp";
    
        
    /** Creates a new instance of DOMAllDataObjectsTimeStamp */
     public DOMAllDataObjectsTimeStamp(
             List<DOMEncapsulatedPKIData>pkidata, 
             String id, 
             Document doc, 
             DOMCryptoContext context)
    throws MarshalException{
                super(
                TAG_ALLDATAOBJECTSTIMESTAMP,
                null, //implicit
                null,
                pkidata,
                null,
                id,
                doc, 
                context);
    }
        
    public DOMAllDataObjectsTimeStamp(Element elem, DOMCryptoContext context) throws MarshalException{
        super(elem, context);
    }
     
   /**Returns the hash using the given message digest algorithm and provider.
    *@param provider may be null.
    *@throws NoSuchAlgorithmException
    *
    */
    public static byte[] getDigest(String algorithm, DOMCryptoContext context, 
            XMLSignature sig, String signedPropertyURI)
     throws NoSuchAlgorithmException, 
            XMLSignatureException, 
            MarshalException{
       if(context instanceof DOMSignContext){
            ((DOMXMLSignature)sig).prepare((DOMSignContext)context, signedPropertyURI);
       }
        
        MessageDigest md = MessageDigest.getInstance(algorithm);   
        DigesterOutputStream dos = new DigesterOutputStream(md);
	OutputStream os = new UnsyncBufferedOutputStream(dos);
        Iterator it = sig.getSignedInfo().getReferences().iterator();
        while(it.hasNext()){
            DOMReference ref = (DOMReference)it.next();
            if(!ref.getURI().equals(signedPropertyURI)){
                transform(ref, context, os);
            }
        }
        return md.digest();
        
    }
    
    private static void transform(DOMReference ref, DOMCryptoContext context, OutputStream os)
    throws XMLSignatureException{
        
        Data data = null;
	if (ref.getAppliedTransformData() == null) {
	    data = ref.dereference(context);
	} else {
	    data = ref.getAppliedTransformData();
	}
        
        List transforms = ref.getTransforms();
        for (int i = 0, size = transforms.size(); i < size; i++) {
	    DOMTransform transform = (DOMTransform) transforms.get(i);
	    try {
                if (i < size - 1) {
                    data = transform.transform(data, context);
                } else {
                    data = transform.transform(data, context, os);
                }
	    } catch (TransformException te) {
		throw new XMLSignatureException(te);
	    }
        }
	
	try {
	    if (data != null) {
	        XMLSignatureInput xi;
	        if (data instanceof ApacheData) {
	            xi = ((ApacheData) data).getXMLSignatureInput();
	        } else if (data instanceof OctetStreamData) {
	            xi = new XMLSignatureInput
			(((OctetStreamData)data).getOctetStream());
	        } else if (data instanceof NodeSetData) {
		    TransformService spi = null;
		    try {
		        spi = TransformService.getInstance
		            (CanonicalizationMethod.INCLUSIVE, "DOM");
		    } catch (NoSuchAlgorithmException nsae) {
		        spi = TransformService.getInstance
		            (CanonicalizationMethod.INCLUSIVE, "DOM", ref.getProvider());
		    }
                    data = spi.transform(data, context);
	            xi = new XMLSignatureInput
		        (((OctetStreamData)data).getOctetStream());
	        } else {
	            throw new XMLSignatureException("unrecognized Data type");
	        }
	        xi.updateOutputStream(os);
	    }
	    os.flush();

      	} catch (Exception e) {
	    throw new XMLSignatureException(e);
	}
    }
}
