/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Attr;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

/**DOM-based implementation of {@code AnyType}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMAnyType extends DOMStructure {
    
    /**The anyType content (node list). May be not null*/
    List<Node> content = new ArrayList<Node>();
    
    /** Creates a new instance of DOMAnyType */
    public DOMAnyType(NodeList content) {
        if(content!=null){
            for(int i=0;i<content.getLength();i++){
                this.content.add(content.item(i));
            }
        }
    }
    
    /** Creates a new instance of DOMAnyType. May be null */
    public DOMAnyType(List<Node> content) {
        if(content!=null){
            this.content = content;
        }
    }
    
    /**Unmarshal.*/
    public DOMAnyType(Element elem, DOMCryptoContext context){
        
        NodeList nodeList = elem.getChildNodes();
        for(int i=0;i<nodeList.getLength();i++){
            this.content.add(nodeList.item(i));
        }
    }
    
    public void marshal(Node parent, String localName, DOMCryptoContext context)
            throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        Element anyElem = DOMUtils.createElement(
                doc, localName, xadesNS, xadesPrefix);
        if(content!=null){
            Iterator<Node> it = content.iterator();
            while(it.hasNext()){
                anyElem.appendChild(it.next());
            }
        }
        parent.appendChild(anyElem);
    }
    
    /**Returns the children nodes of the anyType element. Never null.*/
    public List<Node> getContent() {
        return content;
    }
    
    public String toString(){
        try{
            String dump="";
            Iterator<Node> it = this.getContent().iterator();
            while(it.hasNext()){
                dump += "[Node:\n"+it.next()+"]\n";
            }
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }
    
}
