/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
  */

package be.cardon.xades;

import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Attr;

/**DOM-based implementation of {@code CompleteCertificatesRefsType}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMCompleteCertificateRefs extends DOMStructure{
    
    public final static String TAG_CERTREFS = "CertRefs";
    public final static String TAG_ID = "Id";
    
    /**CertsRefs, never null.*/
    DOMCertIDList certRefs;
    
    /**Id, may be null, but not empty.*/
    String id;
    
    /** Creates a new instance of DOMCompleteCertificateRefs *
     *@param certRefs, never null.
     *@param id, may be null but not empty
     *@throws NullPointerException if certRefs is null.
     *@throws IllegalArgumentException if id is empty.
     */
    public DOMCompleteCertificateRefs(DOMCertIDList certRefs, String id) {
        if(certRefs==null){
            throw new NullPointerException("CertRefs is null");
        }
        if(id!=null && id.length()<1){
            throw new IllegalArgumentException("id is empty.");
        }
        this.certRefs = certRefs;    
    }
    
    /** Unmarshal a DOMCompleteCertificateRefs.*/
    public DOMCompleteCertificateRefs(Element elem, DOMCryptoContext context)
    throws MarshalException{
        try{
        String xadesNS = XadESContext.getXadesNS(context);
        DOMUtils.checkName(elem, null, xadesNS);
        Element certRefsElem = DOMUtils.getFirstChildElement(elem);
        this.certRefs = new DOMCertIDList(certRefsElem, context);
        
        this.id = DOMUtils.getAttributeValue(elem, TAG_ID); //may be null
        
        }catch(NullPointerException e){
            throw new MarshalException(e);
        }
    }

    public void marshal(Node parent, String localName, DOMCryptoContext context)
        throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element completeCertificateRefsElem = DOMUtils.createElement(doc, localName, 
                xadesNS, xadesPrefix);
        this.certRefs.marshal(completeCertificateRefsElem, 
                TAG_CERTREFS, context);
        DOMUtils.setAttributeID(completeCertificateRefsElem, TAG_ID, this.id);
        parent.appendChild(completeCertificateRefsElem);
   }
   
    public DOMCertIDList getCertRefs() {
        return certRefs;
    }

    public String getId() {
        return id;
    }
    
    public String toString(){
        try{   
            String dump = "";
             if(this.getId()!=null){
                dump += "attribute Id="+this.getId()+"\n";
            }
            dump += "[CertRefs:\n"+ this.getCertRefs() +"] end CertRefs\n";
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }

    
}
