/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.dsig.dom.DOMDigestMethod;
import be.cardon.xml.XSDBase64Binary;
import java.security.InvalidAlgorithmParameterException;
import javax.xml.crypto.dsig.DigestMethod;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import javax.xml.crypto.dsig.XMLSignature;
import javax.xml.crypto.dsig.spec.DigestMethodParameterSpec;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**DOM-based implementation of {@code DigestAlgAndValueType}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMDigestAlgAndValue extends DOMStructure{
    
    public final static String TAG_DIGESTMETHOD = "DigestMethod";
    public final static String TAG_DIGESTVALUE = "DigestValue";
    
    /**DigestMethod, never null.*/
    DOMDigestMethod digestMethod;
    
    /**DigestValue, never null.*/
    byte[] digestValue;
    
    /** Creates a new instance of DOMDigestAlgAndValue */
    public DOMDigestAlgAndValue(
            DigestMethod digestMethod,
            byte[] digestValue) {
        if(digestMethod==null){
            throw new NullPointerException("digestMethod cannot be null.");
        }
        if(digestValue==null){
            throw new NullPointerException("digestValue cannot be null.");
        }
        this.digestMethod = (DOMDigestMethod)digestMethod;
        this.digestValue = digestValue;
    }
    
    /**Unmarshal a DigestAlgAndValue element.*/
    public DOMDigestAlgAndValue(Element elem, DOMCryptoContext context)
    throws MarshalException{
        try{
        String xadesNS = XadESContext.getXadesNS(context);
        DOMUtils.checkName(elem, null, xadesNS);
        
        Element digestMethElem = DOMUtils.getFirstChildElement(elem);
        DOMUtils.checkName(digestMethElem, TAG_DIGESTMETHOD, XMLSignature.XMLNS);
        this.digestMethod = (DOMDigestMethod)DOMDigestMethod.unmarshal(
                digestMethElem);
        
        Element digestValue = DOMUtils.getNextSiblingElement(digestMethElem);
        DOMUtils.checkName(digestValue, TAG_DIGESTVALUE, XMLSignature.XMLNS);
        this.digestValue = XSDBase64Binary.decode(digestValue.getTextContent());
        
        }catch(NullPointerException e){
            throw new MarshalException(e);
        }
    }
    
    public void marshal(Node parent, String localName, DOMCryptoContext context)
        throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element digestAlgAndValueElem = DOMUtils.createElement(
                doc, localName, xadesNS, xadesPrefix);
        
        digestMethod.marshal(digestAlgAndValueElem, dsPrefix, context);
        
        Element digestValueElem = DOMUtils.createElement(
                doc, TAG_DIGESTVALUE, XMLSignature.XMLNS,dsPrefix);
        digestValueElem.appendChild(doc.createTextNode(
                XSDBase64Binary.encode(digestValue)));
        digestAlgAndValueElem.appendChild(digestValueElem);
        
        parent.appendChild(digestAlgAndValueElem);
    }

    /**Returns the digest method. Never null.*/
    public DOMDigestMethod getDigestMethod() {
        return digestMethod;
    }

    /**Returns the digest value. Never null.*/
    public byte[] getDigestValue() {
        return digestValue;
    }
    
    public String toString(){
        try{   
            String dump = "";
            dump += "[DigestMethod:\n"+ this.getDigestMethod().toString() +"]\n";
            dump += " DigestValue (base 64): "+ XSDBase64Binary.encode(this.getDigestValue())+"\n";
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }

    

}
