/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.xml.XSDAnyURI;
import java.net.URI;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.crypto.dsig.XMLSignatureException;
import be.cardon.xml.XSDBoolean;

/**DOM-based implementation of {@code IncludeType}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMInclude extends DOMStructure{
    /**URI value. May not be null.*/
    URI uri;
    
    /**referencedData bolean. May be null.*/
    Boolean referencedData;
    
    public final static String TAG_URI = "URI";
    public final static String TAG_REFERENCEDDATA = "ReferencedData";
    
    /**Creates an new instance. referencedData may be null.*
     *throws NullPointerException if uri is null.
     */
    
    public DOMInclude(URI uri, Boolean referencedData){
        if(uri==null){
            throw new NullPointerException("Include URI may not be null.");
        }
        this.uri = uri;
        this.referencedData = referencedData;
    }
    
    /**Unmarshal an Include element.*/
    public DOMInclude(Element elem, DOMCryptoContext context)
    throws MarshalException{
        String URIString = elem.getAttributeNS(null, TAG_URI); //never null
        this.uri = XSDAnyURI.parseURI(URIString);
        if(this.uri==null){
            throw new MarshalException("Include URI attribute absent.");
        }
        String referencedDataString = DOMUtils.getAttributeValue(
                elem, TAG_REFERENCEDDATA); //returns null if absent;
        if(referencedDataString == null){
            this.referencedData = null;
        }else{
            this.referencedData = XSDBoolean.parseBoolean(referencedDataString);
        }

    }
    
    /**Creates an new instance, without referencedData boolean value.*
     *throws NullPointerException if uri is null.
     */
    public DOMInclude(URI uri){
        this(uri, null);
    }
    
    public void marshal(Node parent, String localName,
            DOMCryptoContext context)
            throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element includeElem = DOMUtils.createElement(doc,
                localName, xadesNS, xadesPrefix);
        includeElem.setAttributeNS(null, TAG_URI, uri.toString());
        if(referencedData!=null){
            includeElem.setAttributeNS(null, TAG_REFERENCEDDATA,
                    XSDBoolean.printBoolean(referencedData));
        }
        
        parent.appendChild(includeElem);
    }
    
    /**Returns referenced data boolean. May be null.*/
    public Boolean getReferencedData() {
        return referencedData;
    }
    
    /**Returns URI. Never null.*/
    public URI getUri() {
        return uri;
    }
    
     public String toString(){
        try{   
            String dump = "";
            dump += " URI: " + this.getUri()+ "\n";
            Boolean refData = this.getReferencedData();
            if(refData!=null){
                dump += "ReferencedData: " + refData;
            }
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }
}
