/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.xml.XSDAnyURI;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.net.URI;

/**DOM-based implementation of {@code SPURI}.
 * Warning : SPURI.getContent() returns an empty list when the instance
 * is created with the contructor DOMSPURI(URI sPURI)
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMSPURI extends DOMSigPolicyQualifier{
    
    public static final String TAG_SPURI = "SPURI";
    
    /**SPURI, never null.*/
    URI sPURI;
    
    public DOMSPURI(URI sPURI){
        super((NodeList)null); //bypass DOMAnyType
        if(sPURI==null){
            throw new NullPointerException("sPURI is null.");
        }
        this.sPURI = sPURI;
    }
    
    public DOMSPURI(Element elem, DOMCryptoContext context)
    throws MarshalException{
        super(elem.getChildNodes());
        try{
             String xadesNS = XadESContext.getXadesNS(context);
             if(elem.getChildNodes().getLength()>1){
                throw new MarshalException("SigPolicyQualifier contains more than one child.");
            }
            Element SPURIElem = DOMUtils.getFirstChildElement(elem);
            DOMUtils.checkName(SPURIElem, TAG_SPURI, xadesNS);
            sPURI = XSDAnyURI.parseURI(SPURIElem.getTextContent());
        }catch(NullPointerException e){
            throw new MarshalException(e);
        }
    }
    
    public void marshal(Node parent, 
            String localName, DOMCryptoContext context)
            throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element anyElem = DOMUtils.createElement(
                doc, localName, xadesNS, xadesPrefix);
        Element sPURIElem =  DOMUtils.createElement(
                doc, TAG_SPURI, xadesNS, xadesPrefix);
        sPURIElem.appendChild(doc.createTextNode(XSDAnyURI.printURI(sPURI)));
        anyElem.appendChild(sPURIElem);
        parent.appendChild(anyElem);
    }
    
    /**Returns the signature policy URI. Never null.*/
    public URI getSPURI() {
        return sPURI;
    }
    
    public String toString(){
        try{
        return "SPURI: "+this.getSPURI().toString();
        }catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }
}
