/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
/**DOM-based implementation of {@code PolicyQualifierType}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMSigPolicyQualifier extends DOMAnyType{
    
    public final static String TAG_SPURI = "SPURI";
    public final static String TAG_SPUSERNOTICE = "SPUserNotice";
    
    
    /** Creates a new instance of DOMSigPolicyQualifier */
    public DOMSigPolicyQualifier(NodeList content) {
        super(content);

    }
  
    /** Creates a new instance of DOMAnyType. May be null */
    public DOMSigPolicyQualifier(List<Node> content) {
        super(content);
    }
    
     /** Unmarshall a DisPolicyQualifier. */
    public static DOMSigPolicyQualifier unmarshal(Element elem, DOMCryptoContext context) 
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        DOMUtils.checkName(elem, null, xadesNS);
        try{
        NodeList nl = elem.getChildNodes();
        if(nl.getLength()==1 && nl.item(0).getLocalName().equals(TAG_SPURI)){
            return new DOMSPURI(elem, context);
        }
        if(nl.getLength()==1 && nl.item(0).getLocalName().equals(TAG_SPUSERNOTICE)){
            return new DOMSPUserNotice(elem, context);
        }
        return new DOMSigPolicyQualifier(nl);
        }catch(NullPointerException e){
            throw new MarshalException(e);
        }
        
    }  

}


