/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import org.w3c.dom.*;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;

/**DOM-based implementation of {@code SignatureProductionPlace}.
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMSignatureProductionPlace extends DOMStructure{
    
    public final static String TAG_CITY = "City";
    public final static String TAG_STATEORPROVINCE = "StateOrProvince";
    public final static String TAG_POSTALCODE = "PostalCode";
    public final static String TAG_COUNTRYNAME = "CountryName";
    
    /**Value of the {@code City} element. May be empty, but never null*/
    String city = new String();
    
    /**Value of the {@code StateOrProvince} element. May be empty, but never null*/
    String stateOrProvince = new String();
    
    /**Value of the {@code PostalCode} element. May be empty, but never null*/
    String postalCode = new String();
    
    /**Value of the {@code CountryName} element. May be empty, but never null*/
    String countryName = new String();
    
    /**Creates a new instance. The given data may be empty {@code null}.
     * An empty element is treated as null.
     */
    public DOMSignatureProductionPlace(
            String city, 
            String stateOrProvince, 
            String postalCode,
            String countryName){
        this.city = (city == null) ? new String() : city ;
        this.stateOrProvince = (stateOrProvince==null) ? new String() : stateOrProvince;
        this.postalCode = (postalCode==null) ? new String(): postalCode;
        this.countryName = (countryName==null) ? new String(): countryName;
    }

    /**Returns the city. 
     * A copy is returned. Never {@code null}.*/
    public String getCity() {
        return new String(city);
    }

    /**Returns the country name
     * A copy is returned. Never {@code null}.*/
    public String getCountryName() {
        return new String(countryName);
    }

    /**Returns the portal code
      * A copy is returned. Never {@code null}.*/
    public String getPostalCode() {
        return new String(postalCode);
    }

    /**Returns the state or province
     * A copy is returned. Never {@code null}.*/
    public String getStateOrProvince() {
        return new String(stateOrProvince);
    }

    /** Unmarshal an existing {@code SignatureProductionPlace} XML element. 
     *@param el the element to unmarshall.
     *@param xadesNS the expected XadES namespace.
     *throws MarshalException if invalid format.
     */
    public DOMSignatureProductionPlace(Element el, DOMCryptoContext context) 
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        DOMUtils.checkName(el, null, xadesNS);
        Element child = DOMUtils.getFirstChildElement(el);

        //retrieves the values.
        //Check no duplicate, and correct sequence.
        try{
            
        if(child!=null && child.getLocalName().equals(TAG_CITY)){
            DOMUtils.checkName(child, null, xadesNS);
            city = child.getTextContent(); //never null
            child = DOMUtils.getNextSiblingElement(child);
        }
        if(child!=null && child.getLocalName().equals(TAG_STATEORPROVINCE)){
            DOMUtils.checkName(child, null, xadesNS);
            stateOrProvince = child.getTextContent(); //never null
            child = DOMUtils.getNextSiblingElement(child);
        }
        if(child!=null && child.getLocalName().equals(TAG_POSTALCODE)){
            DOMUtils.checkName(child, null, xadesNS);
            postalCode = child.getTextContent(); //never null
            child = DOMUtils.getNextSiblingElement(child);
        }
        if(child!=null && child.getLocalName().equals(TAG_COUNTRYNAME)){
            DOMUtils.checkName(child, null, xadesNS);
            countryName = child.getTextContent(); //never null
            child = DOMUtils.getNextSiblingElement(child);
        }
        if(child!=null){
            throw new MarshalException(
                    "unknown element in SignatureProductionPlace : "
                    +child.getLocalName());
        }
        
        }catch(NullPointerException e){
            throw new MarshalException(e);
        }
    }

    public void marshal(Node parent, String localName, 
            DOMCryptoContext context)
        throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        
        //don't add an empty SignatureProductionPlace
        if(
                city.length()==0 && 
                stateOrProvince.length()==0 && 
                postalCode.length()==0 &&
                countryName.length()==0){
            return;
        }
        
        Element sppElem = DOMUtils.createElement(doc, localName, xadesNS, xadesPrefix);
        if(city.length()!=0){
            Element cityElem = DOMUtils.createElement(
                    doc, 
                    TAG_CITY, 
                    xadesNS, xadesPrefix);
            cityElem.appendChild(doc.createTextNode(city));
            sppElem.appendChild(cityElem);
        }
        if(stateOrProvince.length()!=0){
            Element sopElem = DOMUtils.createElement(
                    doc, 
                    TAG_STATEORPROVINCE,
                    xadesNS, xadesPrefix);
            sopElem.appendChild(doc.createTextNode(stateOrProvince));
            sppElem.appendChild(sopElem);
        }
        if(postalCode.length()!=0){
            Element pcElem = DOMUtils.createElement(
                    doc,
                    TAG_POSTALCODE,
                    xadesNS, xadesPrefix);
            pcElem.appendChild(doc.createTextNode(postalCode));
            sppElem.appendChild(pcElem);
        }
        if(countryName.length()!=0){
            Element cElem = DOMUtils.createElement(
                    doc,
                    TAG_COUNTRYNAME,
                    xadesNS, xadesPrefix);
            cElem.appendChild(doc.createTextNode(countryName));
            sppElem.appendChild(cElem);
        }
        parent.appendChild(sppElem);
    }

    public String toString(){
           try{   
            String dump = "";
            if(this.getCity().length()>0){
                dump += " City: " + this.getCity() +"\n" ;
            }
            if(this.getStateOrProvince().length()>0){
                dump += " StateOrProvince: " + this.getStateOrProvince() +"\n" ;
            }
            if(this.getPostalCode().length()>0){
                dump += " PostalCode: " + this.getPostalCode() +"\n" ;
            }
            if(this.getCountryName().length()>0){
                dump += " CountryName: " + this.getCountryName() +"\n" ;
            }
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }
}
