/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.crypto.dsig.XMLSignatureException;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
        

/**DOM-based implementation of {@code UnsignedDataObjectProperty}.
 *
 * @author CARDON DE LICHTBUER
 */
public class DOMUnsignedDataObjectProperties extends DOMStructure{
    
    public final static String TAG_UNSIGNEDDATAOBJECTPROPERTY = "UnsignedDataObjectProperty";
    public final static String TAG_ID = "Id";
    
    /**Unsigned data object properties. Never null.*/
    List<DOMAnyType> unsignedDataObjectProperties = new ArrayList<DOMAnyType>();
    
    /**Id. Never empty, may be null.*/
    String id;
    
    /**Creates a new instance of DOMUnsignedDataObjectProperties.
     *@param unsignedDataObjectProperties the unsigned data object properties
     *@param id the Id of the element (may be null, but not empty.
     *@throws IllegaArgumentException if id is empty./
     */
    public DOMUnsignedDataObjectProperties(
            List<DOMAnyType> unsignedDataObjectProperties,
            String id) {
        if(unsignedDataObjectProperties!=null){
            this.unsignedDataObjectProperties = unsignedDataObjectProperties;
        }
        if(id!=null && id.length()<1){
            throw new IllegalArgumentException("id is empty.");
        } 
        this.id = id;
    }
    
    /** Creates a new instance of DOMUnsignedDataObjectProperties */
    public DOMUnsignedDataObjectProperties(Element elem, DOMCryptoContext context) 
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        DOMUtils.checkName(elem, null, xadesNS);
        
        Element unsignedDataObjectPropertyElem = DOMUtils.getFirstChildElement(elem);
        while(unsignedDataObjectPropertyElem !=null){
            DOMUtils.checkName(unsignedDataObjectPropertyElem, 
                    TAG_UNSIGNEDDATAOBJECTPROPERTY, xadesNS);
            this.unsignedDataObjectProperties.add(new DOMAnyType(
                    unsignedDataObjectPropertyElem, context));
        }
        this.id = DOMUtils.getAttributeValue(elem, TAG_ID);
    }
    
  public void marshal(Node parent, 
            String localName,  DOMCryptoContext context)
            throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element unsignedDataObjectPropertiesElem = DOMUtils.createElement(doc,
                localName, xadesNS, xadesPrefix);
       
        Iterator<DOMAnyType> it = unsignedDataObjectProperties.iterator();
        while(it.hasNext()){
            it.next().marshal(unsignedDataObjectPropertiesElem,
                    TAG_UNSIGNEDDATAOBJECTPROPERTY, context);
        }
        DOMUtils.setAttributeID(unsignedDataObjectPropertiesElem, TAG_ID, this.id);
        parent.appendChild(unsignedDataObjectPropertiesElem);
        
  }

    public String getId() {
        return id;
    }

    public List<DOMAnyType> getUnsignedDataObjectProperties() {
        return unsignedDataObjectProperties;
    }

}
