/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import java.security.Provider;
import javax.xml.crypto.dom.DOMCryptoContext;
import javax.xml.crypto.dsig.XMLSignature;

/**Utilities for DOMCryptoContext (xadesNS store)
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class XadESContext{

    public final static String CONTEXT_PROPERTY_XADESNS = "XaDESNameSpace";
    public final static String CONTEXT_PROPERTY_XMLDSIGPROV = "XMLSignatureProvider";
    public final static String XMLDSIG_DEFAULT_PREFIX = "ds";
    
    public static String getXadesNS(DOMCryptoContext context){
           Object obj = context.getProperty(CONTEXT_PROPERTY_XADESNS);
           if(obj!=null){
               return (String)obj;
           }else{
               return Constants.XadesSpecNS;
           }
    }
    public static void setXadesNS(DOMCryptoContext context, String xadesNS){
        context.setProperty(CONTEXT_PROPERTY_XADESNS, xadesNS);
    }
    
    /**Returns the xades default prefix or null if not set.*/
    public static String getXadesNSPrefix(DOMCryptoContext context){
        return context.getNamespacePrefix(
                getXadesNS(context), 
                null);
    }
    
    /**Returns the DSIG default prefix.*/
    public static String getDSIGNSPrefix(DOMCryptoContext context){
        return context.getNamespacePrefix(XMLSignature.XMLNS, XMLDSIG_DEFAULT_PREFIX);
    }
    
    /**Returns the XMLSignature Provider.*/
    public static Provider getXMLSignatureProvider(DOMCryptoContext context){
        Provider prov = (Provider)context.getProperty(CONTEXT_PROPERTY_XMLDSIGPROV);
        if(prov==null){
            return new be.cardon.xades.XadESProvider();
        }else{
            return prov;
        }
    }   
    /**set the XMLSignature Provider.*/
    public static void setXMLSignatureProvider(DOMCryptoContext context, Provider XMLSigProv){
        context.setProperty(CONTEXT_PROPERTY_XMLDSIGPROV, XMLSigProv);
    }   
}
