/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.xades;

import java.util.*;
import java.security.*;
import javax.xml.crypto.dsig.*;


/*The XMLSignature provider for XadES.
 *
 *@author CARDON DE LICHTBUER Rodolphe
 */
public class XadESProvider extends java.security.Provider{

    //static final long serialVersionUID = -5049765099299494554L;

    private static final String INFO = "XMLDSig " + 
    "(DOM XMLSignatureFactory; DOM KeyInfoFactory)";

    public XadESProvider() {
	/* We are the XMLDSig provider */
        super("XadES", 1.0, "XML Advanced Electronic Signature (by WebSign project)"); 
	
        String prefixPackage = "be.cardon.dsig.dom.";
        
	final Map map = new HashMap();
        map.put("XMLSignatureFactory.DOM", 
	        prefixPackage + "DOMXMLSignatureFactory");
        map.put("KeyInfoFactory.DOM", 
	        prefixPackage + "DOMKeyInfoFactory");


	// Inclusive C14N
	map.put((String)"TransformService." + CanonicalizationMethod.INCLUSIVE,
		prefixPackage + "DOMCanonicalXMLC14NMethod");
	map.put("Alg.Alias.TransformService.INCLUSIVE", 
		CanonicalizationMethod.INCLUSIVE);
	map.put((String)"TransformService." + CanonicalizationMethod.INCLUSIVE +
		" MechanismType", "DOM");

	// InclusiveWithComments C14N
	map.put((String) "TransformService." + 
		CanonicalizationMethod.INCLUSIVE_WITH_COMMENTS,
		prefixPackage + "DOMCanonicalXMLC14NMethod");
	map.put("Alg.Alias.TransformService.INCLUSIVE_WITH_COMMENTS", 
		CanonicalizationMethod.INCLUSIVE_WITH_COMMENTS);
	map.put((String) "TransformService." + 
		CanonicalizationMethod.INCLUSIVE_WITH_COMMENTS + 
		" MechanismType", "DOM");

	// Exclusive C14N
	map.put((String) "TransformService." + CanonicalizationMethod.EXCLUSIVE,
		prefixPackage + "DOMExcC14NMethod");
	map.put("Alg.Alias.TransformService.EXCLUSIVE", 
		CanonicalizationMethod.EXCLUSIVE);
	map.put((String)"TransformService." + CanonicalizationMethod.EXCLUSIVE +
		" MechanismType", "DOM");

	// ExclusiveWithComments C14N
	map.put((String) "TransformService." + 
		CanonicalizationMethod.EXCLUSIVE_WITH_COMMENTS,
		prefixPackage + "DOMExcC14NMethod");
	map.put("Alg.Alias.TransformService.EXCLUSIVE_WITH_COMMENTS", 
		CanonicalizationMethod.EXCLUSIVE_WITH_COMMENTS);
	map.put((String) "TransformService." + 
		CanonicalizationMethod.EXCLUSIVE_WITH_COMMENTS +
		" MechanismType", "DOM");

	// Base64 Transform
	map.put((String) "TransformService." + Transform.BASE64,
		prefixPackage + "DOMBase64Transform");
	map.put("Alg.Alias.TransformService.BASE64", Transform.BASE64);
	map.put((String) "TransformService." + Transform.BASE64 + 
		" MechanismType", "DOM");

	// Enveloped Transform
	map.put((String) "TransformService." + Transform.ENVELOPED,
		prefixPackage + "DOMEnvelopedTransform");
	map.put("Alg.Alias.TransformService.ENVELOPED", Transform.ENVELOPED);
	map.put((String) "TransformService." + Transform.ENVELOPED + 
		" MechanismType", "DOM");

	// XPath2 Transform
	map.put((String) "TransformService." + Transform.XPATH2,
		prefixPackage + "DOMXPathFilter2Transform");
	map.put("Alg.Alias.TransformService.XPATH2", Transform.XPATH2);
	map.put((String) "TransformService." + Transform.XPATH2 + 
		" MechanismType", "DOM");

	// XPath Transform
	map.put((String) "TransformService." + Transform.XPATH,
		prefixPackage + "DOMXPathTransform");
	map.put("Alg.Alias.TransformService.XPATH", Transform.XPATH);
	map.put((String) "TransformService." + Transform.XPATH + 
		" MechanismType", "DOM");

	// XSLT Transform
	map.put((String) "TransformService." + Transform.XSLT,
		prefixPackage + "DOMXSLTTransform");
	map.put("Alg.Alias.TransformService.XSLT", Transform.XSLT);
	map.put((String) "TransformService." + Transform.XSLT + 
		" MechanismType", "DOM");

	AccessController.doPrivileged(new java.security.PrivilegedAction() {
	    public Object run() {
		putAll(map);
		return null;
	    }
	});
    }
}
